from django.contrib.auth.models import Group
from rest_framework import viewsets

from .serializers import ConsumerSerializer, GroupSerializer
from .models import Consumer


class ConsumerViewSet(viewsets.ReadOnlyModelViewSet):

    serializer_class = ConsumerSerializer
    lookup_field = 'kong_consumer_id'

    def get_queryset(self):
        user = self.request.user
        if user.is_anonymous:
            return Consumer.objects.none()
        elif user.is_staff:
            return Consumer.objects.all()
        else:
            return Consumer.objects.filter(pk=self.request.user.pk)

class GroupViewSet(viewsets.ReadOnlyModelViewSet):

    serializer_class = GroupSerializer
    lookup_field = 'id'

    def get_queryset(self):
        return Group.objects.all()



from django.urls import reverse
from rest_framework import status
from rest_framework.test import APITestCase
from django.contrib.auth.models import Group

from ..models import Consumer


class ComsumersTests(APITestCase):
    def setUp(self):
        self.staff_consumer_id = 'a1-a1'
        self.staff_consumer = Consumer.objects.create_user(self.staff_consumer_id, is_staff=True)

        self.custom_consumer_id = 'b1-b1'
        self.custom_consumer = Consumer.objects.create_user(self.custom_consumer_id)


    def test_create_consumer(self):
        """
        Ensure we can create a new consumer object.
        """
        url = reverse('consumer-list')

        kong_consumer_id = 'b2-b2'
        group = 'example.com'

        headers = {
            'HTTP_X_CONSUMER_USERNAME': kong_consumer_id,
            'HTTP_X_CONSUMER_GROUPS': group
        }

        response = self.client.get(url, **headers, format='json')

        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(Consumer.objects.count(), 3)
        self.assertEqual(Consumer.objects.filter(kong_consumer_id=kong_consumer_id).count(), 1)
        self.assertEqual(Group.objects.filter(name=group).count(), 1)


    def test_create_staff_consumer(self):
        """
        Ensure we can create a new staff consumer object.
        """
        url = reverse('consumer-list')

        kong_consumer_id = 'a2-a2'
        group = 'staff'

        headers = {
            'HTTP_X_CONSUMER_USERNAME': kong_consumer_id,
            'HTTP_X_CONSUMER_GROUPS': group
        }

        response = self.client.get(url, **headers, format='json')

        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(Consumer.objects.count(), 3)
        self.assertEqual(Consumer.objects.filter(kong_consumer_id=kong_consumer_id).count(), 1)
        self.assertEqual(Group.objects.filter(name=group).count(), 0)


    def test_staff_consumer_can_list_all_users(self):
        """
        Ensure we can create a new staff consumer object.
        """
        url = reverse('consumer-list')

        kong_consumer_id = 'a2-a2'
        group = 'staff'

        headers = {
            'HTTP_X_CONSUMER_USERNAME': kong_consumer_id,
            'HTTP_X_CONSUMER_GROUPS': group
        }

        response = self.client.get(url, **headers, format='json')
        response_data = response.json()
        self.assertEqual(len(response_data), 3)


    def test_custom_consumer_can_list_only_self(self):
        """
        Ensure we can create a new staff consumer object.
        """
        url = reverse('consumer-list')

        kong_consumer_id = 'b2-b2'
        group = 'example.com'

        headers = {
            'HTTP_X_CONSUMER_USERNAME': kong_consumer_id,
            'HTTP_X_CONSUMER_GROUPS': group
        }

        response = self.client.get(url, **headers, format='json')
        response_data = response.json()
        self.assertEqual(len(response_data), 1)


    def test_custom_consumer_can_get_only_self(self):
        """
        Ensure we can create a new staff consumer object.
        """

        kong_consumer_id = 'b2-b2'
        group = 'example.com'

        headers = {
            'HTTP_X_CONSUMER_USERNAME': kong_consumer_id,
            'HTTP_X_CONSUMER_GROUPS': group
        }

        url = reverse('consumer-detail', args=(kong_consumer_id, ))

        response = self.client.get(url, **headers, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        response_data = response.json()
        self.assertEqual(response_data.get('kong_consumer_id'), kong_consumer_id)

        url2 = reverse('consumer-detail', args=(self.custom_consumer_id, ))

        response2 = self.client.get(url2, **headers, format='json')
        self.assertEqual(response2.status_code, status.HTTP_404_NOT_FOUND)


    def test_staff_consumer_can_get_different_user(self):
        """
        Ensure we can create a new staff consumer object.
        """

        kong_consumer_id = 'a2-a2'
        group = 'staff'

        headers = {
            'HTTP_X_CONSUMER_USERNAME': kong_consumer_id,
            'HTTP_X_CONSUMER_GROUPS': group
        }

        url = reverse('consumer-detail', args=(kong_consumer_id, ))

        response = self.client.get(url, **headers, format='json')
        response_data = response.json()
        self.assertEqual(response_data.get('kong_consumer_id'), kong_consumer_id)

        url2 = reverse('consumer-detail', args=(self.custom_consumer_id, ))

        response2 = self.client.get(url2, **headers, format='json')
        self.assertEqual(response2.status_code, status.HTTP_200_OK)

        response2_data = response2.json()
        self.assertEqual(response2_data.get('kong_consumer_id'), self.custom_consumer_id)


    def test_can_update_groups(self):
        """
        Ensure we can create a new staff consumer object.
        """

        kong_consumer_id = 'b2-b2'
        group = 'example.com'

        headers = {
            'HTTP_X_CONSUMER_USERNAME': kong_consumer_id,
            'HTTP_X_CONSUMER_GROUPS': group
        }

        url = reverse('consumer-detail', args=(kong_consumer_id, ))

        response = self.client.get(url, **headers, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        response_data = response.json()
        self.assertEqual(response_data.get('kong_consumer_id'), kong_consumer_id)

        group2 = 'example.org, staff'
        headers2 = {
            'HTTP_X_CONSUMER_USERNAME': kong_consumer_id,
            'HTTP_X_CONSUMER_GROUPS': group2
        }

        url2 = reverse('consumer-detail', args=(kong_consumer_id, ))

        response2 = self.client.get(url2, **headers2, format='json')
        self.assertEqual(response2.status_code, status.HTTP_200_OK)

        self.assertEqual(Consumer.objects.get(kong_consumer_id=kong_consumer_id).is_staff, True)
        self.assertEqual(Consumer.objects.get(kong_consumer_id=kong_consumer_id).groups.count(), 1)
        self.assertEqual(Consumer.objects.get(kong_consumer_id=kong_consumer_id).groups.filter(name=group).count(), 0)
        self.assertEqual(Consumer.objects.get(kong_consumer_id=kong_consumer_id).groups.filter(name='example.org').count(), 1)



from django.contrib.auth.models import Group
from rest_framework import serializers

from .models import Consumer


class ConsumerSerializer(serializers.ModelSerializer):
    groups = serializers.SlugRelatedField(
        many=True,
        read_only=True,
        slug_field='name')

    class Meta:
        model = Consumer
        fields = ('kong_consumer_id', 'created_at', 'updated_at', 'groups')

class GroupSerializer(serializers.ModelSerializer):
    class Meta:
        model = Group
        fields = ('id', 'name')



import os
from django.conf import settings


def env_to_bool(name, default=False):
    var = os.environ.get(name, '')
    res = default
    if default == True:
        if var in ['false', 'False', 0]:
            res = False
    else:
        if var in ['true', 'True', 1]:
            res = True
    return res


KONG_CONSUMER_ID_HEADER = getattr(settings, 'KONG_CONSUMER_ID_HEADER', os.environ.get('KONG_CONSUMER_ID_HEADER', 'HTTP_X_CONSUMER_USERNAME'))
KONG_ANONYMOUS_CONSUMER_ID = getattr(settings, 'KONG_ANONYMOUS_CONSUMER_ID', os.environ.get('KONG_ANONYMOUS_CONSUMER_ID', 'anonymous'))
KONG_ANONYMOUS_CONSUMER_HEADER = getattr(settings, 'KONG_ANONYMOUS_CONSUMER_HEADER', os.environ.get('KONG_ANONYMOUS_CONSUMER_HEADER', 'HTTP_X_ANONYMOUS_COMSUMER'))
KONG_CREATE_CONSUMER_GROUPS = getattr(settings, 'KONG_CREATE_CONSUMER_GROUPS', env_to_bool('KONG_CREATE_CONSUMER_GROUPS', True))



import os
import sys
import django

from django.conf import settings
from django.test.utils import get_runner


def runtests():
    os.environ['DJANGO_SETTINGS_MODULE'] = 'django_rest_framework_kong_consumers.tests.test_settings'
    django.setup()
    TestRunner = get_runner(settings)
    test_runner = TestRunner()
    failures = test_runner.run_tests(["django_rest_framework_kong_consumers.tests"])
    sys.exit(bool(failures))


if __name__ == '__main__':
    runtests()


from django.db import models
from django.contrib.auth import models as auth_models
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone


class ConsumerManager(auth_models.BaseUserManager):

    def create_user(self, kong_consumer_id, password=None, **extra_fields):
        """
        Creates and saves a User with the given username, email and
        password.
        """
        now = timezone.now()

        if not kong_consumer_id:
            raise ValueError('Users must have a kong_consumer_id')

        consumer = self.model(
            kong_consumer_id=kong_consumer_id,

            is_active=True,
            is_superuser=False,
            last_login=now, created_at=now, **extra_fields)

        if password:
            consumer.set_password(password)

        consumer.save(using=self._db)
        return consumer

    def create_superuser(self, kong_consumer_id, password, **extra_fields):
        u = self.create_user(kong_consumer_id, password, **extra_fields)
        u.is_staff = True
        u.is_active = True
        u.is_superuser = True
        u.save(using=self._db)
        return u


class Consumer(auth_models.AbstractBaseUser, auth_models.PermissionsMixin):
    is_staff = models.BooleanField(
        _('Staff status'), default=False,
        help_text=_('Designates whether the user can log into this admin '
                    'site.'))
    is_active = models.BooleanField(
        _('Active'), default=True,
        help_text=_('Designates whether this user should be treated as '
                    'active. Unselect this instead of deleting accounts.'))

    kong_consumer_id = models.CharField(max_length=36, unique=True)

    created_at = models.DateTimeField(_('date joined'), default=timezone.now)
    updated_at = models.DateTimeField(_('date updated'), null=True, blank=True)

    objects = ConsumerManager()

    USERNAME_FIELD = 'kong_consumer_id'
    REQUIRED_FIELDS = []

    @property
    def classname(self):
        return self.__class__.__name__.lower()

    @property
    def is_alive(self):
        return self.is_active

    class Meta:
        verbose_name = _('Consumer')
        verbose_name_plural = _('Consumers')

    def get_full_name(self):
        return self.kong_consumer_id

    def get_short_name(self):
        return self.get_full_name()

    def __str__(self):
        return self.get_full_name()



import logging
from rest_framework import authentication
from django.contrib.auth.models import AnonymousUser
from django.contrib.auth.models import Group
from django.utils import timezone

from .models import Consumer
from .functions import extract_groups

from .settings import (KONG_CONSUMER_ID_HEADER,
                       KONG_ANONYMOUS_CONSUMER_ID,
                       KONG_ANONYMOUS_CONSUMER_HEADER,
                       KONG_CREATE_CONSUMER_GROUPS)


logger = logging.getLogger('django_rest_framework_kong_consumers.authentication')

def update_consumer_groups(user, groups):
    if KONG_CREATE_CONSUMER_GROUPS:
        for group in groups:
            if group != 'staff':
                g, _created = Group.objects.get_or_create(name=group)
                logger.info('Created group {}'.format(g))
                g.user_set.add(user)

def get_kong_user(request):

    is_anonymous = request.META.get(KONG_ANONYMOUS_CONSUMER_HEADER, None) is not None
    kong_consumer_id = request.META.get(KONG_CONSUMER_ID_HEADER, KONG_ANONYMOUS_CONSUMER_ID)
    groups = extract_groups(request.META.get('HTTP_X_CONSUMER_GROUPS', 'users'))
    is_staff = ('staff' in groups)

    logger.debug('Got kong_consumer_id: {}'.format(kong_consumer_id))

    # Consumer is anonymous
    if is_anonymous or kong_consumer_id == KONG_ANONYMOUS_CONSUMER_ID:
        logger.info('Kong says user is anonymous')
        return
      
    # Consumer is custom
    try:
        user = Consumer.objects.get(kong_consumer_id=kong_consumer_id)

    except Consumer.DoesNotExist:
        logger.info('kong_consumer_id: {} does not exists, creating with groups {}'.format(kong_consumer_id, groups))
        user = Consumer.objects.create_user(kong_consumer_id, is_staff=is_staff)
        update_consumer_groups(user, groups)
    else:
        _updated = False

        g1 = list(map(lambda x: x.name, user.groups.only('name')))
        g2 = list(filter(lambda x: x != 'staff', groups))
        if g1 != g2:
            logger.info('Groups updated from {} to {}'.format(g1, g2))
            user.groups.clear()
            update_consumer_groups(user, groups)
            _updated = True

        if is_staff and not user.is_staff:
            user.is_staff = True
            _updated = True

        if _updated:
            user.updated_at = timezone.now()
            user.save()

    return user


class KongConsumerAuthentication(authentication.BaseAuthentication):
    def authenticate(self, request):
        user = get_kong_user(request)
        logger.info('Athenticated as {}'.format(user))
        return (user, None)



import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = 'fake-key'

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    
    'rest_framework',
    'django_rest_framework_kong_consumers',
]

ROOT_URLCONF = 'django_rest_framework_kong_consumers.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
    },
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

AUTH_USER_MODEL = 'django_rest_framework_kong_consumers.Consumer'

REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': [
        'django_rest_framework_kong_consumers.authentication.KongConsumerAuthentication'
    ]
}

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'simple': {
            'format': '%(levelname)s:%(name)s: %(message)s '
                      '(%(asctime)s; %(filename)s:%(lineno)d)',
            'datefmt': "%Y-%m-%d %H:%M:%S",
        },
    },
    'handlers': {
        'console':{
            'level':'DEBUG',
            'class':'logging.StreamHandler',
            'formatter': 'simple'
        }
    },
    'loggers': {
        'django': {
            'level': 'INFO',
            'handlers': ['console'],
            'propagate': True,
        },
        'django_rest_framework_kong_consumers': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': True,
        },
    },
}



from django.conf.urls import url

from .views import ConsumerViewSet, GroupViewSet


urlpatterns = [
    url(r'^consumers/$', ConsumerViewSet.as_view({'get': 'list'}), name="consumer-list"),
    url(r'^consumers/(?P<kong_consumer_id>[0-9A-Fa-f-]+)/$', ConsumerViewSet.as_view({'get': 'retrieve'}), name="consumer-detail"),

    url(r'^groups/$', GroupViewSet.as_view({'get': 'list'}), name="group-list"),
]
